@@ -1,4 +1,4 @@
-/* This is part of the netCDF package. Copyright 2005 University
+/* This is part of the netCDF package. Copyright 2018 University
  * Corporation for Atmospheric Research/Unidata. See \ref copyright
  * file for conditions of use. */
 /** @internal @file
@@ -17,11 +17,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include "nc_logging.h"
 #ifdef USE_PARALLEL
 #include "netcdf_par.h"
 #endif
 #include "netcdf.h"
-//#include "err_macros.h"
 
 /** NC_MAX_DIMS for tests.  Allows different NC_MAX_DIMS values
  * without breaking this test with a heap or stack overflow. */
@@ -41,6 +41,7 @@
 #define ONE_TWENTY_EIGHT_MEG (SIXTEEN_MEG * 8)
 #define TEST_VAL_42 42
 #define BAD_NAME "dd//d/  "
+#define NUM_CLASSIC_TYPES 6
 #define NUM_NETCDF_TYPES 12
 /** \} */
 
