@@ -1,8 +1,8 @@
-/*! \file
-Functions for defining and inquiring about variables.
-
-Copyright 2010 University Corporation for Atmospheric
-Research/Unidata. See COPYRIGHT file for more info.
+/* Copyright 2010-2018 University Corporation for Atmospheric
+   Research/Unidata. See COPYRIGHT file for more info. */
+/**
+ * @file
+ * Functions for defining and inquiring about variables.
 */
 
 #include "ncdispatch.h"
@@ -149,7 +149,8 @@ nc_inq_ncid().
 @param ndims Number of dimensions for the variable. For example, 2
 specifies a matrix, 1 specifies a vector, and 0 means the variable is
 a scalar with no dimensions. Must not be negative or greater than the
-predefined constant ::NC_MAX_VAR_DIMS.
+predefined constant ::NC_MAX_VAR_DIMS. In netCDF-4/HDF5 files, may not
+exceed the HDF5 maximum number of dimensions (32).
 
 @param dimidsp Vector of ndims dimension IDs corresponding to the
 variable dimensions. For classic model netCDF files, if the ID of the
@@ -239,6 +240,10 @@ For classic format, 64-bit offset format, and netCDF-4/HDF5 with
 classic mode, if the new name is longer than the old name, the netCDF
 dataset must be in define mode.
 
+For netCDF-4/HDF5 files, renaming the variable changes the order of
+the variables in the file. The renamed variable becomes the last
+variable in the file.
+
 @param ncid NetCDF or group ID, from a previous call to nc_open(),
 nc_create(), nc_def_grp(), or associated inquiry functions such as
 nc_inq_ncid().
@@ -279,7 +284,7 @@ rel_hum in an existing netCDF dataset named foo.nc:
      status = nc_enddef(ncid);
      if (status != NC_NOERR) handle_error(status);
 @endcode
-
+@author Glenn Davis, Ed Hartnett, Dennis Heimbigner
 */
 int
 nc_rename_var(int ncid, int varid, const char *name)
@@ -416,13 +421,16 @@ NC_inq_recvar(int ncid, int varid, int* nrecdimsp, int *is_recdim)
    entirely.
 */
 
-/** \internal
-\ingroup variables
-Find the length of a type. This is how much space is required by the user, as in
-\code
-vals = malloc(nel * nctypelen(var.type));
-ncvarget(cdfid, varid, cor, edg, vals);
-\endcode
+/**
+ * @internal
+ * @ingroup variables
+ * Find the length of a type. This is how much space is required by
+ * the in memory to hold one element of this type.
+ *
+ * @param type A netCDF atomic type.
+ *
+ * @return Length of the type in bytes, or -1 if type not found.
+ * @author Ed Hartnett
  */
 int
 nctypelen(nc_type type)
@@ -464,7 +472,7 @@ nctypelen(nc_type type)
 
 /** \internal
 \ingroup variables
-Find the length of a type. Redunant over nctypelen() above. */
+Find the length of a type. Redundant over nctypelen() above. */
 size_t
 NC_atomictypelen(nc_type xtype)
 {
@@ -538,6 +546,190 @@ NC_getshape(int ncid, int varid, int ndims, size_t* shape)
    return status;
 }
 
+/**
+ * @ingroup variables
+ *
+ * Set the fill value for a variable.
+ *
+ * @note For netCDF classic, 64-bit offset, and CDF5 formats, it is
+ * allowed (but not good practice) to set the fill value after data
+ * have been written to the variable. In this case, unless the
+ * variable has been completely specified (without gaps in the data),
+ * any existing filled values will not be recognized as fill values by
+ * applications reading the data. Best practice is to set the fill
+ * value after the variable has been defined, but before any data have
+ * been written to that varibale. In NetCDF-4 files, this is enforced
+ * by the HDF5 library. For netCDF-4 files, an error is returned if
+ * the user attempts to set the fill value after writing data to the
+ * variable.
+
+ * @param ncid NetCDF ID, from a previous call to nc_open or
+ * nc_create.
+ * @param varid Variable ID.
+ * @param no_fill Set to NC_NOFILL to turn off fill mode for this
+ * variable. Set to NC_FILL (the default) to turn on fill mode for the
+ * variable.
+ * @param fill_value the fill value to be used for this variable. Must
+ * be the same type as the variable. This must point to enough free
+ * memory to hold one element of the data type of the variable. (For
+ * example, an NC_INT will require 4 bytes for it's fill value, which
+ * is also an NC_INT.)
+ *
+ * @returns ::NC_NOERR No error.
+ * @returns ::NC_EBADID Bad ID.
+ * @returns ::NC_ENOTINDEFINE Not in define mode.  This is returned
+ * for netCDF classic, 64-bit offset, or 64-bit data files, or for
+ * netCDF-4 files, when they were created with NC_STRICT_NC3 flag. See
+ * @ref nc_create.
+ * @returns ::NC_EPERM Attempt to create object in read-only file.
+ * @returns ::NC_ELATEDEF (NetCDF-4 only). Returned when user attempts
+ * to set fill value after data are written.
+ * @returns ::NC_EGLOBAL Attempt to set fill value on NC_GLOBAL.
+ *
+ * @section nc_def_var_fill_example Example
+ *
+ * In this example from libsrc4/tst_vars.c, a variable is defined, and
+ * the fill mode turned off. Then nc_inq_fill() is used to check that
+ * the setting is correct. Then some data are written to the
+ * variable. Since the data that are written do not cover the full
+ * extent of the variable, the missing values will just be random. If
+ * fill value mode was turned on, the missing values would get the
+ * fill value.
+ *
+ @code
+#define DIM7_LEN 2
+#define DIM7_NAME "dim_7_from_Indiana"
+#define VAR7_NAME "var_7_from_Idaho"
+#define NDIMS 1
+      int dimids[NDIMS];
+      size_t index[NDIMS];
+      int varid;
+      int no_fill;
+      unsigned short ushort_data = 42, ushort_data_in, fill_value_in;
+
+      if (nc_create(FILE_NAME, NC_NETCDF4, &ncid)) ERR;
+      if (nc_def_dim(ncid, DIM7_NAME, DIM7_LEN, &dimids[0])) ERR;
+      if (nc_def_var(ncid, VAR7_NAME, NC_USHORT, NDIMS, dimids,
+		     &varid)) ERR;
+      if (nc_def_var_fill(ncid, varid, 1, NULL)) ERR;
+
+      if (nc_inq_var_fill(ncid, varid, &no_fill, &fill_value_in)) ERR;
+      if (!no_fill) ERR;
+
+      index[0] = 1;
+      if (nc_put_var1_ushort(ncid, varid, index, &ushort_data)) ERR;
+
+      index[0] = 0;
+      if (nc_get_var1_ushort(ncid, varid, index, &ushort_data_in)) ERR;
+
+      if (nc_close(ncid)) ERR;
+ @endcode
+ * @author Glenn Davis, Ed Hartnett, Dennis Heimbigner
+*/
+int
+nc_def_var_fill(int ncid, int varid, int no_fill, const void *fill_value)
+{
+    NC* ncp;
+    int stat = NC_check_id(ncid,&ncp);
+    if(stat != NC_NOERR) return stat;
+
+    /* Dennis Heimbigner: Using NC_GLOBAL is illegal, as this API has no
+     * provision for specifying the type of the fillvalue, it must of necessity
+     * be using the type of the variable to interpret the bytes of the
+     * fill_value argument.
+     */
+    if (varid == NC_GLOBAL) return NC_EGLOBAL;
+
+    return ncp->dispatch->def_var_fill(ncid,varid,no_fill,fill_value);
+}
+
+/**
+ * @internal Check the start, count, and stride parameters for gets
+ * and puts, and handle NULLs.
+ *
+ * @param ncid The file ID.
+ * @param varid The variable ID.
+ * @param start Pointer to start array. If NULL NC_EINVALCOORDS will
+ * be returned for non-scalar variable.
+ * @param count Pointer to pointer to count array. If *count is NULL,
+ * an array of the correct size will be allocated, and filled with
+ * counts that represent the full extent of the variable. In this
+ * case, the memory must be freed by the caller.
+ * @param stride Pointer to pointer to stride array. If NULL, stide is
+ * ignored. If *stride is NULL an array of the correct size will be
+ * allocated, and filled with ones. In this case, the memory must be
+ * freed by the caller.
+ *
+ * @return ::NC_NOERR No error.
+ * @return ::NC_EBADID Bad ncid.
+ * @return ::NC_ENOTVAR Variable not found.
+ * @return ::NC_ENOMEM Out of memory.
+ * @return ::NC_EINVALCOORDS Missing start array.
+ * @author Ed Hartnett
+ */
+int
+NC_check_nulls(int ncid, int varid, const size_t *start, size_t **count,
+               ptrdiff_t **stride)
+{
+   int varndims;
+   int stat;
+
+   if ((stat = nc_inq_varndims(ncid, varid, &varndims)))
+      return stat;
+
+   /* For non-scalar vars, start is required. */
+   if (!start && varndims)
+      return NC_EINVALCOORDS;
+
+   /* If count is NULL, assume full extent of var. */
+   if (!*count)
+   {
+      if (!(*count = malloc(varndims * sizeof(size_t))))
+         return NC_ENOMEM;
+      if ((stat = NC_getshape(ncid, varid, varndims, *count)))
+      {
+         free(*count);
+         *count = NULL;
+         return stat;
+      }
+   }
+
+   /* If stride is NULL, do nothing, if *stride is NULL use all 1s. */
+   if (stride && !*stride)
+   {
+      int i;
+
+      if (!(*stride = malloc(varndims * sizeof(ptrdiff_t))))
+         return NC_ENOMEM;
+      for (i = 0; i < varndims; i++)
+         (*stride)[i] = 1;
+   }
+
+   return NC_NOERR;
+}
+
+/** \ingroup variables
+Free string space allocated by the library.
+
+When you read string type the library will allocate the storage space
+for the data. This storage space must be freed, so pass the pointer
+back to this function, when you're done with the data, and it will
+free the string memory.
+
+\param len The number of character arrays in the array.
+\param data The pointer to the data array.
+
+\returns ::NC_NOERR No error.
+*/
+int
+nc_free_string(size_t len, char **data)
+{
+   int i;
+   for (i = 0; i < len; i++)
+      free(data[i]);
+   return NC_NOERR;
+}
+
 #ifdef USE_NETCDF4
 /** \ingroup variables
 
@@ -650,28 +842,6 @@ nc_get_var_chunk_cache(int ncid, int varid, size_t *sizep, size_t *nelemsp,
 					      nelemsp, preemptionp);
 }
 
-/** \ingroup variables
-Free string space allocated by the library.
-
-When you read string type the library will allocate the storage space
-for the data. This storage space must be freed, so pass the pointer
-back to this function, when you're done with the data, and it will
-free the string memory.
-
-\param len The number of character arrays in the array.
-\param data The pointer to the data array.
-
-\returns ::NC_NOERR No error.
-*/
-int
-nc_free_string(size_t len, char **data)
-{
-   int i;
-   for (i = 0; i < len; i++)
-      free(data[i]);
-   return NC_NOERR;
-}
-
 /**
  * @ingroup variables
  *
@@ -680,11 +850,22 @@ nc_free_string(size_t len, char **data)
  * This function must be called after nc_def_var and before nc_enddef
  * or any functions which writes data to the file.
  *
+ * Deflation and shuffline require chunked data. If this function is
+ * called on a variable with contiguous data, then the data is changed
+ * to chunked data, with default chunksizes. Use nc_def_var_chunking()
+ * to tune performance with user-defined chunksizes.
+ *
+ * If this function is called on a scalar variable, it is ignored.
+ *
  * @param ncid NetCDF or group ID, from a previous call to nc_open(),
  * nc_create(), nc_def_grp(), or associated inquiry functions such as
  * nc_inq_ncid().
  * @param varid Variable ID
- * @param shuffle True to turn on the shuffle filter.
+ * @param shuffle True to turn on the shuffle filter. The shuffle
+ * filter can assist with the compression of integer data by changing
+ * the byte order in the data stream. It makes no sense to use the
+ * shuffle filter without setting a deflate level, or to use shuffle
+ * on non-integer data.
  * @param deflate True to turn on deflation for this variable.
  * @param deflate_level A number between 0 (no compression) and 9
  * (maximum compression).
@@ -752,6 +933,7 @@ filter and compression.
       ERR(retval);
         ...
 @endcode
+* @author Ed Hartnett, Dennis Heimbigner
 */
 int
 nc_def_var_deflate(int ncid, int varid, int shuffle, int deflate, int deflate_level)
@@ -770,6 +952,11 @@ nc_def_var_deflate(int ncid, int varid, int shuffle, int deflate, int deflate_le
  * This function must be called after nc_def_var and before nc_enddef
  * or any functions which writes data to the file.
  *
+ * Checksums require chunked data. If this function is called on a
+ * variable with contiguous data, then the data is changed to chunked
+ * data, with default chunksizes. Use nc_def_var_chunking() to tune
+ * performance with user-defined chunksizes.
+ *
  * @param ncid NetCDF or group ID, from a previous call to nc_open(),
  * nc_create(), nc_def_grp(), or associated inquiry functions such as
  * nc_inq_ncid().
@@ -786,6 +973,7 @@ not netCDF-4/HDF5.
 netcdf-4 file.
  * @returns ::NC_ELATEDEF Too late to change settings for this variable.
  * @returns ::NC_EINVAL Invalid input
+ * @author Ed Hartnett, Dennis Heimbigner
 */
 int
 nc_def_var_fletcher32(int ncid, int varid, int fletcher32)
@@ -899,80 +1087,6 @@ nc_def_var_chunking(int ncid, int varid, int storage,
 					   chunksizesp);
 }
 
-/*! Set the fill value for a netCDF4/HDF5 variable.
-
-\ingroup variables
-
-\param ncid NetCDF ID, from a previous call to nc_open or
-nc_create.
-
-\param varid Variable ID.
-
-\param no_fill Set to NC_NOFILL to turn off fill mode for this
-variable. Set to NC_FILL (the default) to turn on fill mode for the
-variable.
-
-\param fill_value the fill value to be used for this variable. Must be
-the same type as the variable. This must point to enough free memory
-to hold one element of the data type of the variable. (For example, an
-NC_INT will require 4 bytes for it's fill value, which is also an
-NC_INT.)
-
- * @returns ::NC_NOERR No error.
- * @returns ::NC_EBADID Bad ID.
- * @returns ::NC_ENOTNC4 Not a netCDF-4 file.
- * @returns ::NC_ENOTINDEFINE Not in define mode.  This is returned for
-netCDF classic or 64-bit offset files, or for netCDF-4 files, when
-they wwere created with NC_STRICT_NC3 flag. See \ref nc_create.
- * @returns ::NC_EPERM Attempt to create object in read-only file.
-
-\section nc_def_var_fill_example Example
-
-In this example from libsrc4/tst_vars.c, a variable is defined, and
-the fill mode turned off. Then nc_inq_fill() is used to check that the
-setting is correct. Then some data are written to the variable. Since
-the data that are written do not cover the full extent of the
-variable, the missing values will just be random. If fill value mode
-was turned on, the missing values would get the fill value.
-
-\code
-#define DIM7_LEN 2
-#define DIM7_NAME "dim_7_from_Indiana"
-#define VAR7_NAME "var_7_from_Idaho"
-#define NDIMS 1
-      int dimids[NDIMS];
-      size_t index[NDIMS];
-      int varid;
-      int no_fill;
-      unsigned short ushort_data = 42, ushort_data_in, fill_value_in;
-
-      if (nc_create(FILE_NAME, NC_NETCDF4, &ncid)) ERR;
-      if (nc_def_dim(ncid, DIM7_NAME, DIM7_LEN, &dimids[0])) ERR;
-      if (nc_def_var(ncid, VAR7_NAME, NC_USHORT, NDIMS, dimids,
-		     &varid)) ERR;
-      if (nc_def_var_fill(ncid, varid, 1, NULL)) ERR;
-
-      if (nc_inq_var_fill(ncid, varid, &no_fill, &fill_value_in)) ERR;
-      if (!no_fill) ERR;
-
-      index[0] = 1;
-      if (nc_put_var1_ushort(ncid, varid, index, &ushort_data)) ERR;
-
-      index[0] = 0;
-      if (nc_get_var1_ushort(ncid, varid, index, &ushort_data_in)) ERR;
-
-      if (nc_close(ncid)) ERR;
-\endcode
-*/
-int
-nc_def_var_fill(int ncid, int varid, int no_fill, const void *fill_value)
-{
-    NC* ncp;
-    int stat = NC_check_id(ncid,&ncp);
-    if(stat != NC_NOERR) return stat;
-    return ncp->dispatch->def_var_fill(ncid,varid,no_fill,fill_value);
-}
-
 /**
 @ingroup variables
 
@@ -984,8 +1098,11 @@ same as the default endianness of the platform. But with
 nc_def_var_endianness the endianness can be explicitly set for a
 variable.
 
+Warning: this function is only defined if the type of the variable
+is an atomic integer or float type.
+
 This function may only be called after the variable is defined, but
-before nc_enddef is called. 
+before nc_enddef is called.
 
 @param[in] ncid NetCDF ID, from a previous call to nc_open or
 nc_create.
@@ -1059,6 +1176,7 @@ nc_def_var_endian(int ncid, int varid, int endian)
  * @param parms Filter parameters.
  *
  * @return ::NC_NOERR No error.
+ * @return ::NC_EBADID Bad ID.
  * @author Dennis Heimbigner
  */
 int
