@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 2010, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *   $Header$
  *********************************************************************/
@@ -67,7 +67,7 @@ static char* queryallow =
 static char* userpwdallow =
 "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!$&'()*+,-.;=_~?#/";
 
-#ifndef HAVE_STRNCMP
+#ifndef HAVE_STRNDUP
 #define strndup ncstrndup
 /* Not all systems have strndup, so provide one*/
 char*
@@ -191,7 +191,7 @@ ncuriparse(const char* uri0, NCURI** durip)
 	    }
 	} else { /*c == '#'*/
 	    tmp.fragment = next;
-	}	    
+	}
     }
 
     /* Parse the prefix parameters */
@@ -217,7 +217,7 @@ ncuriparse(const char* uri0, NCURI** durip)
 	    nclistpush(querylist,NULL);
             tmp.querylist = nclistextract(querylist);
 	}
-    }    
+    }
 
     /* Now parse the core of the url */
     p = tmp.uri;
@@ -245,7 +245,7 @@ ncuriparse(const char* uri0, NCURI** durip)
        Note in all cases, the leading '/' is considered part of the path,
        which is then assumed to be an absolute path. But also note that
        the windows drive letter has to be taken into account. Our rule is that
-       if the path looks like D:..., 
+       if the path looks like D:...,
        where D is a single alphabetic letter (a-z or A-Z),
        then it is a windows path and can be use in place of a /path.
        The rules implemented here (for file:) are then as follows
@@ -258,12 +258,12 @@ ncuriparse(const char* uri0, NCURI** durip)
 
     isfile = (strcmp(tmp.protocol,"file")==0);
     if(isfile) {
-	int l = strlen(p); /* to test if we have enough characters */
+	size_t l = strlen(p); /* to test if we have enough characters */
 	hashost = 0; /* always */
 	if(l >= 2 && p[1] == ':' && strchr(DRIVELETTERS,p[0]) != NULL) { /* case 1 */
-	    p = p; /* p points to the start of the path */
+	    ; /* p points to the start of the path */
         } else if(l >= 2 && p[0] == '/' && p[1] != '/') { /* case 2 */
-	    p = p; /* p points to the start of the path */
+	    ; /* p points to the start of the path */
 	} else if(l >= 4 && p[0] == '/' && p[1] == '/'
 		&& p[3] == ':' && strchr(DRIVELETTERS,p[2]) != NULL) { /* case 3 */
 	    p = p+2; /* points to the start of the windows path */
@@ -319,7 +319,7 @@ ncuriparse(const char* uri0, NCURI** durip)
 		{THROW(NCU_EUSRPWD);} /* we have empty user */
 	    if(strlen(pp)==0)
 		{THROW(NCU_EUSRPWD);} /* we have empty password */
-	    tmp.password = pp;	    
+	    tmp.password = pp;
 	    tmp.host = newhost;
 	}
 	/* Breakup host into host + port */
@@ -351,7 +351,7 @@ ncuriparse(const char* uri0, NCURI** durip)
     /* before saving, we need to decode the user+pwd */
     duri->user = NULL;
     duri->password = NULL;
-    if(tmp.user != NULL) 
+    if(tmp.user != NULL)
         duri->user = ncuridecode(tmp.user);
     if(tmp.password != NULL)
         duri->password = ncuridecode(tmp.password);
@@ -368,7 +368,10 @@ ncuriparse(const char* uri0, NCURI** durip)
     duri->fragment = nulldup(tmp.fragment);
     duri->fraglist = tmp.fraglist; tmp.fraglist = NULL;
     duri->querylist = tmp.querylist; tmp.querylist = NULL;
-    if(durip) *durip = duri;
+    if(durip)
+      *durip = duri;
+    else
+      free(duri);
 
 #ifdef NCXDEBUG
 	{
@@ -387,7 +390,8 @@ ncuriparse(const char* uri0, NCURI** durip)
 
 done:
     if(uri != NULL)
-	free(uri);
+      free(uri);
+
     freestringlist(params);
     freestringlist(querylist);
     freestringvec(tmp.fraglist);
@@ -412,10 +416,10 @@ static void
 freestringvec(char** list)
 {
     if(list != NULL) {
-	char** p;	
+	char** p;
         for(p=list;*p;p++) {nullfree(*p);}
 	nullfree(list);
-    }    
+    }
 }
 
 void
@@ -450,7 +454,7 @@ int
 ncurisetquery(NCURI* duri,const char* query)
 {
     int ret = NCU_OK;
-    freestringvec(duri->querylist);    
+    freestringvec(duri->querylist);
     nullfree(duri->query);
     duri->query = NULL;
     duri->querylist = NULL;
@@ -560,7 +564,7 @@ ncuribuild(NCURI* duri, const char* prefix, const char* suffix, int flags)
 	    char* encoded = ncuriencodeonly(duri->path,pathallow);
 	    ncbytescat(buf,encoded);
 	    nullfree(encoded);
-	} else 	
+	} else
 	    ncbytescat(buf,duri->path);
     }
 
@@ -582,10 +586,10 @@ ncuribuild(NCURI* duri, const char* prefix, const char* suffix, int flags)
 		    char* encoded = ncuriencodeonly(p[1],queryallow);
 		    ncbytescat(buf,encoded);
 	            nullfree(encoded);
-		} else 	
+		} else
 		    ncbytescat(buf,p[1]);
 	    }
-	}	
+	}
     }
     if((flags & NCURIFRAG) && duri->fraglist != NULL) {
 	char** p;
@@ -599,10 +603,10 @@ ncuribuild(NCURI* duri, const char* prefix, const char* suffix, int flags)
 		    char* encoded = ncuriencodeonly(p[1],queryallow);
 		    ncbytescat(buf,encoded);
 	            nullfree(encoded);
-		} else 	
+		} else
 		    ncbytescat(buf,p[1]);
 	    }
-	}	
+	}
     }
     ncbytesnull(buf);
     newuri = ncbytesextract(buf);
@@ -647,7 +651,7 @@ ncuriremoveparam(NCURI* uri, const char* key)
     if(uri->fraglist == NULL) return NCU_OK;
     for(q=uri->fraglist,p=uri->fraglist;*p;) {
         if(strcmp(key,*p)==0) {
-	    p += 2; /* skip this entry */	
+	    p += 2; /* skip this entry */
 	} else {
 	    *q++ = *p++; /* move key */
 	    *q++ = *p++; /* move value */
@@ -658,14 +662,16 @@ ncuriremoveparam(NCURI* uri, const char* key)
 #endif
 
 
-/* Internal version of lookup; returns the paired index of the key */
+/* Internal version of lookup; returns the paired index of the key;
+   case insensitive
+ */
 static int
 ncfind(char** params, const char* key)
 {
     int i;
     char** p;
     for(i=0,p=params;*p;p+=2,i++) {
-	if(strcmp(key,*p)==0) return i;
+	if(strcasecmp(key,*p)==0) return i;
     }
     return -1;
 }
@@ -901,7 +907,7 @@ collectprefixparams(char* text, char** nextp)
 	last = ep; /* save this position  */
 	ep++; /* move past rbracket */
 	sp = ep;
-    }	
+    }
     /* nul terminate */
     if(last != NULL)
 	terminate(last);
@@ -912,7 +918,7 @@ collectprefixparams(char* text, char** nextp)
 	char* p; char* q;
 	/* by construction, here we are at an LBRACKET: compress it out */
 	for(p=sp,q=sp+1;(*p++=*q++);)
-	    ;	
+	    ;
         /* locate the next RRACKET */
         ep = nclocate(sp,RBRACKETSTR);
 	if(ep == NULL) break;/* we are done */
@@ -920,9 +926,9 @@ collectprefixparams(char* text, char** nextp)
 	*ep = '&';
 	ep++; /* move past rbracket */
 	sp = ep;
-    }	
+    }
 done:
-    return ret;    
+    return ret;
 }
 
 static int
@@ -943,10 +949,10 @@ parselist(char* ptext, NClist* list)
 	if(ep != NULL) {
 	    terminate(ep); /* overwrite the trailing ampersand */
 	    p = ep+1; /* next param */
-	}	
+	}
 	/* split into key + value */
         eq = strchr(sp,'=');
-        if(eq != NULL) { /* value is present */	    
+        if(eq != NULL) { /* value is present */
 	    terminate(eq); eq++;
 	    key = strdup(sp);
 	    value = strdup(eq);
