@@ -57,7 +57,7 @@ hdf4_rec_grp_del(NC_GRP_INFO_T *grp)
       NC_VAR_HDF4_INFO_T *hdf4_var;
       
       var = (NC_VAR_INFO_T *)ncindexith(grp->vars, i);
-      if (var == NULL) continue;
+      assert(var);
       LOG((4, "%s: deleting var %s", __func__, var->hdr.name));
 
       /* Get the HDF4 specific var metadata. */
@@ -67,7 +67,6 @@ hdf4_rec_grp_del(NC_GRP_INFO_T *grp)
        * scale. */
       if (hdf4_var->sdsid && SDendaccess(hdf4_var->sdsid) < 0)
          return NC_EHDFERR;
-      free(hdf4_var);
    }
 
    return NC_NOERR;
@@ -416,6 +415,7 @@ nc4_var_list_add_full(NC_GRP_INFO_T* grp, const char* name, int ndims, nc_type x
    (*var)->created = NC_TRUE;
    (*var)->written_to = NC_TRUE;
    (*var)->format_var_info = format_var_info;
+   (*var)->atts_read = 1;
 
    /* Fill special type_info struct for variable type information. */
    if ((retval = nc4_set_var_type(xtype, endianness, type_size, type_name,
@@ -631,6 +631,7 @@ NC_HDF4_open(const char *path, int mode, int basepe, size_t *chunksizehintp,
    assert(nc4_info && nc4_info->root_grp);
    h5 = nc4_info;
    h5->no_write = NC_TRUE;
+   h5->root_grp->atts_read = 1;
 
    /* Allocate data to hold HDF4 specific file data. */
    if (!(hdf4_file = malloc(sizeof(NC_HDF4_FILE_INFO_T))))
@@ -663,7 +664,7 @@ NC_HDF4_open(const char *path, int mode, int basepe, size_t *chunksizehintp,
 }
 
 /**
- * @internal Close the HDF4 file.
+ * @internal Abort (close) the HDF4 file.
  *
  * @param ncid File ID.
  *
@@ -672,7 +673,23 @@ NC_HDF4_open(const char *path, int mode, int basepe, size_t *chunksizehintp,
  * @author Ed Hartnett
  */
 int
-NC_HDF4_close(int ncid)
+NC_HDF4_abort(int ncid)
+{
+   return NC_HDF4_close(ncid, NULL);
+}
+
+/**
+ * @internal Close the HDF4 file.
+ *
+ * @param ncid File ID.
+ * @param ignore Ignore this pointer.
+ *
+ * @return ::NC_NOERR No error.
+ * @return ::NC_EBADID Bad ncid.
+ * @author Ed Hartnett
+ */
+int
+NC_HDF4_close(int ncid, void *ignore)
 {
    NC_GRP_INFO_T *grp;
    NC *nc;
