@@ -1,7 +1,7 @@
 /**
  * @internal
  *
- * Copyright 2003, University Corporation for Atmospheric
+ * Copyright 2018, University Corporation for Atmospheric
  * Research. See netcdf-4/docs/COPYRIGHT file for copying and
  * redistribution conditions.
  *
@@ -17,37 +17,6 @@
 #include "nc4internal.h"
 #include "nc4dispatch.h"
 
-/**
- * @internal This function only does anything for netcdf-3 files.
- *
- * @param ncid File ID (ignored).
- * @param pe Processor element (ignored).
- *
- * @return ::NC_ENOTNC3 Not a netCDF classic format file.
- * @author Ed Hartnett
- */
-int
-NC4_set_base_pe(int ncid, int pe)
-{
-   return NC_ENOTNC3;
-}
-
-/**
- * @internal This function only does anything for netcdf-3 files.
- *
- * @param ncid File ID (ignored).
- * @param pe Pointer to processor element. Ignored if NULL. Gets a 0
- * if present.
- *
- * @return ::NC_ENOTNC3 Not a netCDF classic format file.
- * @author Ed Hartnett
- */
-int
-NC4_inq_base_pe(int ncid, int *pe)
-{
-   return NC_ENOTNC3;
-}
-
 /**
  * @internal Get the format (i.e. NC_FORMAT_NETCDF4 pr
  * NC_FORMAT_NETCDF4_CLASSIC) of an open netCDF-4 file.
@@ -62,8 +31,8 @@ NC4_inq_base_pe(int ncid, int *pe)
 int
 NC4_inq_format(int ncid, int *formatp)
 {
-   NC *nc;
-   NC_HDF5_FILE_INFO_T* nc4_info;
+   NC_FILE_INFO_T *nc4_info;
+   int retval;
 
    LOG((2, "nc_inq_format: ncid 0x%x", ncid));
 
@@ -71,11 +40,10 @@ NC4_inq_format(int ncid, int *formatp)
       return NC_NOERR;
 
    /* Find the file metadata. */
-   if (!(nc = nc4_find_nc_file(ncid,&nc4_info)))
-      return NC_EBADID;
+   if ((retval = nc4_find_nc_grp_h5(ncid, NULL, NULL, &nc4_info)))
+      return retval;
 
-   /* Otherwise, this is a netcdf-4 file. Check if classic NC3 rules
-    * are in effect for this file. */
+   /* Check if classic NC3 rules are in effect for this file. */
    if (nc4_info->cmode & NC_CLASSIC_MODEL)
       *formatp = NC_FORMAT_NETCDF4_CLASSIC;
    else
@@ -92,8 +60,7 @@ NC4_inq_format(int ncid, int *formatp)
  * @param formatp a pointer that gets the extended format. Note that
  * this is not the same as the format provided by nc_inq_format(). The
  * extended foramt indicates the dispatch layer model. NetCDF-4 files
- * will always get NC_FORMATX_NC4 for netCDF files, NC_FORMATX_HDF4
- * for HDF4 files.
+ * will always get NC_FORMATX_NC4.
  * @param modep a pointer that gets the open/create mode associated with
  * this file. Ignored if NULL.
 
@@ -105,23 +72,18 @@ int
 NC4_inq_format_extended(int ncid, int *formatp, int *modep)
 {
    NC *nc;
-   NC_HDF5_FILE_INFO_T* h5;
+   int retval;
 
-   LOG((2, "nc_inq_format_extended: ncid 0x%x", ncid));
+   LOG((2, "%s: ncid 0x%x", __func__, ncid));
 
-   /* Find the file metadata. */
-   if (!(nc = nc4_find_nc_file(ncid,&h5)))
+   if ((retval = nc4_find_nc_grp_h5(ncid, &nc, NULL, NULL)))
       return NC_EBADID;
 
-   if(modep) *modep = (nc->mode|NC_NETCDF4);
+   if(modep)
+      *modep = nc->mode|NC_NETCDF4;
 
-   if(formatp) {
-#ifdef USE_HDF4
-      /* Distinguish HDF5 from HDF4 */
-      *formatp = (h5->hdf4 ? NC_FORMATX_NC_HDF4 : NC_FORMATX_NC_HDF5);
-#else /* USE_HDF4 */
+   if (formatp)
       *formatp = NC_FORMATX_NC_HDF5;
-#endif /* USE_HDF4 */
-   }
+
    return NC_NOERR;
 }
